@@ -62,7 +62,6 @@ from .sql.sqltypes import POINT as POINT
 
 
 from .sql.clauses import Alias as Alias
-from .sql.clauses import Count as Count
 from .sql.clauses import Delete as Delete
 from .sql.clauses import GroupBy as GroupBy
 from .sql.clauses import Insert as Insert
@@ -76,12 +75,9 @@ from .sql.clauses import Having as Having
 from .sql.clauses import Update as Update
 from .sql.clauses import Upsert as Upsert
 
-from .sql.functions import Max as Max
-from .sql.functions import Min as Min
-from .sql.functions import Concat as Concat
-from .sql.functions import Sum as Sum
-
+from .sql import functions as functions
 
+from .sql.functions import *  # noqa: F403
 from . import util as _util
 
 _util.import_prefix("ormlambda")
