@@ -18,16 +18,16 @@ class UnmatchedLambdaParameterError(Exception):
         return f"Unmatched number of parameters in lambda function with the number of tables: Expected {self.expected_params} parameters but found {str(self.found_param)}."
 
 
-class NotKeysInIAggregateError(Exception):
+class NotKeysInIFunctionError(Exception):
     def __init__(self, match_regex: list[str], *args: object) -> None:
         super().__init__(*args)
         self._match_regex: list[str] = match_regex
 
     def __str__(self) -> str:
-        return f"We cannot use placeholders in IAggregate class. You used {self._match_regex}"
+        return f"We cannot use placeholders in IFunction class. You used {self._match_regex}"
 
 
-class AggregateFunctionError[T](Exception):
+class FunctionFunctionError[T](Exception):
     def __init__(self, clause: ClauseInfo[T], *args):
         self.clause = clause
         super().__init__(*args)
@@ -36,18 +36,18 @@ class AggregateFunctionError[T](Exception):
         agg_methods = self.__get_all_aggregate_method(self.clause)
         return f"You cannot use aggregation method like '{agg_methods}' to return model objects. Try specifying 'flavour' attribute as 'dict'."
 
-    @util.preload_module("ormlambda.sql.clause_info")
+    @util.preload_module("ormlambda.sql.functions")
     def __get_all_aggregate_method(self, clauses: list[ClauseInfo]) -> str:
         """
-        Get the class name of those classes that inherit from 'IAggregate' class in order to create a better error message.
+        Get the class name of those classes that inherit from 'IFunction' class in order to create a better error message.
         """
 
-        IAggregate = util.preloaded.sql_clause_info.IAggregate
+        IFunction = util.preloaded.sql_functions.IFunction 
         res: set[str] = set()
         if not isinstance(clauses, tp.Iterable):
             return clauses.__class__.__name__
         for clause in clauses:
-            if isinstance(clause, IAggregate):
+            if isinstance(clause, IFunction):
                 res.add(clause.__class__.__name__)
         return ", ".join(res)
 
@@ -56,5 +56,5 @@ class NotCallableError(ValueError):
     def __init__(self, *args):
         super().__init__(*args)
 
-    def __str__(self)->str:
-        return f"You must provide a function or callable to proceed with the query creation. Passed '{self.args[0].__class__.__name__}' "
\ No newline at end of file
+    def __str__(self) -> str:
+        return f"You must provide a function or callable to proceed with the query creation. Passed '{self.args[0].__class__.__name__}' "
