@@ -17,10 +17,10 @@ class NoSuchModuleError(Exception):
         return f"NoSuchModuleError: {self.args[0]}"
 
 
-class DuplicatedClauseName(Exception):
+class DuplicatedClauseNameError(Exception):
     def __init__(self, names: tuple[str], **kw):
         self.names = names
         super().__init__(**kw)
 
     def __str__(self):
-        return f"Some clauses has the same alias. {self.names}\nTry wrapping the clause with the 'Alias' class first or setting 'avoid_duplicate' param as 'True'"
+        return f"Some clauses has the same alias. {self.names}\nTry wrapping the clause with the 'Alias' class first or setting 'avoid_duplicates' param as 'True'"
