@@ -6,9 +6,11 @@ from abc import abstractmethod
 if TYPE_CHECKING:
     from ormlambda import Table
     from ormlambda.sql.types import TupleJoinType, ColumnType
+    from ormlambda.sql.types import compileOptions
+    from ormlambda.sql.types import AliasType
+
     from ormlambda.sql.clauses.join import JoinContext
     from ormlambda.common.enums import JoinType
-    from ormlambda.sql.types import AliasType
 
     from ..types import OrderTypes
     from ..types import Tuple
@@ -16,6 +18,7 @@ if TYPE_CHECKING:
     from ..types import WhereTypes
     from ..types import SelectCols
 
+
 from ormlambda.sql.elements import Element
 
 
@@ -172,7 +175,7 @@ class IStatements[T: Table](Element):
     @overload
     def order[TValue](self, columns: Iterable[SelectCols[T, TValue]], order_type: Iterable[OrderTypes]) -> IStatements[T]: ...
     @abstractmethod
-    def order[TValue](self, columns: SelectCols[T, TValue], order_type: OrderTypes = ...) -> IStatements[T]: ...
+    def order[TValue](self, columns, order_type=...) -> IStatements[T]: ...
 
     # endregion
 
@@ -238,22 +241,22 @@ class IStatements[T: Table](Element):
     @overload
     def select_one[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
     @overload
-    def select_one(self, selector: Callable[[T], tuple], *, flavour: Type[dict], by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> dict[str, Any]: ...
+    def select_one[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., flavour: Type[tuple], avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
     @overload
-    def select_one[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, flavour: Type[tuple], by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
+    def select_one[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., flavour: Type[list], avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
     @overload
-    def select_one[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, flavour: Type[list], by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
+    def select_one[TFlavour](self, selector: Callable[[T], Any], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., flavour: Type[TFlavour], avoid_duplicates: bool = ...) -> TFlavour: ...
     @overload
-    def select_one[TFlavour, *TRes](self, selector: Callable[[T], tuple[*TRes]], *, flavour: Type[TFlavour], by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> TFlavour[*TRes]: ...
+    def select_one(self, selector: Callable[[T], Any], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., flavour: Type[dict], avoid_duplicates: bool = ...) -> dict[str, Any]: ...
 
     @abstractmethod
     def select_one(
         self,
         selector=...,
         *,
-        flavour=...,
         by=...,
         alias=...,
+        flavour=...,
         avoid_duplicates=...,
     ): ...
 
@@ -265,22 +268,22 @@ class IStatements[T: Table](Element):
     @overload
     def first[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
     @overload
-    def first(self, selector: Callable[[T], tuple], *, flavour: Type[dict], by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> dict[str, Any]: ...
+    def first[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., flavour: Type[tuple], avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
     @overload
-    def first[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, flavour: Type[tuple], by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
+    def first[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., flavour: Type[list], avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
     @overload
-    def first[*TRes](self, selector: Callable[[T], tuple[*TRes]], *, flavour: Type[list], by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> tuple[*TRes]: ...
+    def first[TFlavour](self, selector: Callable[[T], Any], *, by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., flavour: Type[TFlavour], avoid_duplicates: bool = ...) -> TFlavour: ...
     @overload
-    def first[TFlavour, *TRes](self, selector: Callable[[T], tuple[*TRes]], *, flavour: Type[TFlavour], by: JoinType = ..., alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> TFlavour[*TRes]: ...
+    def first(self, selector: Callable[[T], tuple], *, by: JoinType = ..., flavour: Type[dict], alias: Optional[AliasType[ColumnType]] = ..., avoid_duplicates: bool = ...) -> dict[str, Any]: ...
 
     @abstractmethod
     def first(
         self,
         selector=...,
         *,
-        flavour=...,
         by=...,
         alias=...,
+        flavour=...,
         avoid_duplicates=...,
     ): ...
 
@@ -294,6 +297,7 @@ class IStatements[T: Table](Element):
     @abstractmethod
     def groupby[TRepo](self, column: list[SelectCols[T, TRepo]] | SelectCols[T, TRepo]) -> IStatements[T]: ...
 
-    def compile(self) -> str: ...
+    @abstractmethod
+    def query(self, component: Optional[compileOptions]) -> str: ...
 
     # endregion
